<?php
/**
 * Recreate ActionScheduler tables if missing.
 *
 * @since      1.2.3
 * @package    RevivePress
 * @subpackage RevivePress\Tools
 * @author     Sayan Datta <iamsayan@protonmail.com>
 */

namespace RevivePress\Tools;

use RevivePress\Helpers\Ajax;
use RevivePress\Helpers\Hooker;

defined( 'ABSPATH' ) || exit;

/**
 * Database Table class.
 */
class DatabaseTable
{
	use Ajax, Hooker;

	/**
	 * Register functions.
	 */
	public function register() {
		$this->ajax( 'process_fix_database_tables', 'maybe_recreate_actionscheduler_tables' );
	}

	/**
	 * Recreate ActionScheduler tables if missing.
	 */
	public function maybe_recreate_actionscheduler_tables() {
		global $wpdb;

		$this->verify_nonce();

		if ( $this->is_woocommerce_active() ) {
			return;
		}

		if (
			! class_exists( 'ActionScheduler_HybridStore' )
			|| ! class_exists( 'ActionScheduler_StoreSchema' )
			|| ! class_exists( 'ActionScheduler_LoggerSchema' )
		) {
			return;
		}

		$table_list = [
			'actionscheduler_actions',
			'actionscheduler_logs',
			'actionscheduler_groups',
			'actionscheduler_claims',
		];

		$found_tables = $wpdb->get_col( "SHOW TABLES LIKE '{$wpdb->prefix}actionscheduler%'" );
		foreach ( $table_list as $table_name ) {
			if ( ! in_array( $wpdb->prefix . $table_name, $found_tables, true ) ) {
				$this->recreate_actionscheduler_tables();
				break;
			}
		}

		$this->success();
	}

	/**
	 * Force the data store schema updates.
	 */
	private function recreate_actionscheduler_tables() {
		$store = new \ActionScheduler_HybridStore(); // @phpstan-ignore-line
		add_action( 'action_scheduler/created_table', [ $store, 'set_autoincrement' ], 10, 2 );

		$store_schema  = new \ActionScheduler_StoreSchema(); // @phpstan-ignore-line
		$logger_schema = new \ActionScheduler_LoggerSchema(); // @phpstan-ignore-line
		$store_schema->register_tables( true ); // @phpstan-ignore-line
		$logger_schema->register_tables( true ); // @phpstan-ignore-line

		remove_action( 'action_scheduler/created_table', [ $store, 'set_autoincrement' ], 10 );
	}

	/**
	 * Is WooCommerce Installed
	 *
	 * @return bool
	 */
	private function is_woocommerce_active() {
		// @codeCoverageIgnoreStart
		if ( ! function_exists( 'is_plugin_active' ) ) {
			include_once ABSPATH . 'wp-admin/includes/plugin.php';
		}
		// @codeCoverageIgnoreEnd
		return is_plugin_active( 'woocommerce/woocommerce.php' ) && function_exists( 'is_woocommerce' );
	}
}